@@ -185,6 +185,37 @@ public class AWSClientUtil extends AWSGeneralUtil {
                     final T clientBuilder,
                     final String awsUserAgentPrefixFormat,
                     final String awsClientUserAgentPrefix) {
+        return createAwsSyncClient(
+                configProps,
+                httpClient,
+                clientBuilder,
+                ClientOverrideConfiguration.builder(),
+                awsUserAgentPrefixFormat,
+                awsClientUserAgentPrefix);
+    }
+
+    /**
+     * Creates an AWS Sync Client.
+     *
+     * @param configProps configuration properties
+     * @param httpClient the underlying HTTP client used to talk to AWS
+     * @param clientBuilder the builder for the AWS SDK client
+     * @param clientOverrideConfigurationBuilder the builder for custom override configuration
+     * @param awsUserAgentPrefixFormat user agent prefix for Flink
+     * @param awsClientUserAgentPrefix user agent prefix for kinesis client
+     * @return a new AWS Sync Client
+     */
+    public static <
+                    S extends SdkClient,
+                    T extends
+                            AwsSyncClientBuilder<? extends T, S> & AwsClientBuilder<? extends T, S>>
+            S createAwsSyncClient(
+                    final Properties configProps,
+                    final SdkHttpClient httpClient,
+                    final T clientBuilder,
+                    final ClientOverrideConfiguration.Builder clientOverrideConfigurationBuilder,
+                    final String awsUserAgentPrefixFormat,
+                    final String awsClientUserAgentPrefix) {
         SdkClientConfiguration clientConfiguration = SdkClientConfiguration.builder().build();
 
         String flinkUserAgentPrefix =
@@ -194,7 +225,7 @@ public class AWSClientUtil extends AWSGeneralUtil {
         final ClientOverrideConfiguration overrideConfiguration =
                 createClientOverrideConfiguration(
                         clientConfiguration,
-                        ClientOverrideConfiguration.builder(),
+                        clientOverrideConfigurationBuilder,
                         flinkUserAgentPrefix);
 
         updateEndpointOverride(configProps, clientBuilder);
