@@ -25,6 +25,7 @@ import org.apache.flink.util.ExceptionUtils;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import software.amazon.awssdk.arns.Arn;
 import software.amazon.awssdk.auth.credentials.AwsBasicCredentials;
 import software.amazon.awssdk.auth.credentials.AwsCredentialsProvider;
 import software.amazon.awssdk.auth.credentials.DefaultCredentialsProvider;
@@ -296,9 +297,8 @@ public class AWSGeneralUtil {
         return createAsyncHttpClient(configProperties, NettyNioAsyncHttpClient.builder());
     }
 
-    public static SdkAsyncHttpClient createAsyncHttpClient(
-            final Properties configProperties,
-            final NettyNioAsyncHttpClient.Builder httpClientBuilder) {
+    @VisibleForTesting
+    static AttributeMap getSdkHttpConfigurationOptions(final Properties configProperties) {
         final AttributeMap.Builder clientConfiguration =
                 AttributeMap.builder().put(SdkHttpConfigurationOption.TCP_KEEPALIVE, true);
 
@@ -334,7 +334,15 @@ public class AWSGeneralUtil {
                         protocol ->
                                 clientConfiguration.put(
                                         SdkHttpConfigurationOption.PROTOCOL, protocol));
-        return createAsyncHttpClient(clientConfiguration.build(), httpClientBuilder);
+
+        return clientConfiguration.build();
+    }
+
+    public static SdkAsyncHttpClient createAsyncHttpClient(
+            final Properties configProperties,
+            final NettyNioAsyncHttpClient.Builder httpClientBuilder) {
+        return createAsyncHttpClient(
+                getSdkHttpConfigurationOptions(configProperties), httpClientBuilder);
     }
 
     public static SdkAsyncHttpClient createAsyncHttpClient(
@@ -354,12 +362,29 @@ public class AWSGeneralUtil {
         return httpClientBuilder.buildWithDefaults(config.merge(HTTP_CLIENT_DEFAULTS));
     }
 
+    public static SdkHttpClient createSyncHttpClient(
+            final Properties configProperties, final ApacheHttpClient.Builder httpClientBuilder) {
+        return createSyncHttpClient(
+                getSdkHttpConfigurationOptions(configProperties), httpClientBuilder);
+    }
+
     public static SdkHttpClient createSyncHttpClient(
             final AttributeMap config, final ApacheHttpClient.Builder httpClientBuilder) {
         httpClientBuilder.connectionAcquisitionTimeout(CONNECTION_ACQUISITION_TIMEOUT);
         return httpClientBuilder.buildWithDefaults(config.merge(HTTP_CLIENT_DEFAULTS));
     }
 
+    /**
+     * Extract region from resource ARN.
+     *
+     * @param arn resource ARN
+     * @return An {@link Optional} containing region name (e.g. us-east-1), or an empty {@link
+     *     Optional} if ARN does not contain region component
+     */
+    public static Optional<String> getRegionFromArn(final String arn) {
+        return Arn.fromString(arn).region();
+    }
+
     /**
      * Creates a {@link Region} object from the given Properties.
      *
